﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using MindFusion.Charting.Wpf;
using System.Collections.Specialized;

namespace MultiAxis
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        public Window1()
        {
            InitializeComponent();

            lineChart.XAxes.CollectionChanged += new NotifyCollectionChangedEventHandler(OnAxesCollectionChanged);
            lineChart.YAxes.CollectionChanged += new NotifyCollectionChangedEventHandler(OnAxesCollectionChanged);
            lineChart.X2Axes.CollectionChanged += new NotifyCollectionChangedEventHandler(OnAxesCollectionChanged);
            lineChart.Y2Axes.CollectionChanged += new NotifyCollectionChangedEventHandler(OnAxesCollectionChanged);

            InitializeChart();
            InitializeSettingsGrid();
        }

        private void InitializeChart()
        {
            lineChart.XAxes.Clear();
            lineChart.X2Axes.Clear();
            lineChart.YAxes.Clear();
            lineChart.Y2Axes.Clear();

            lineChart.XAxes.Add(CreateAxis(lineChart.XAxes));
            lineChart.X2Axes.Add(CreateAxis(lineChart.X2Axes));
            lineChart.YAxes.Add(CreateAxis(lineChart.YAxes));
            lineChart.Y2Axes.Add(CreateAxis(lineChart.Y2Axes));

            radio1.IsChecked = false;
            radio1.IsChecked = true;
        }

        private void OnAxesCollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            UpdateSeries();
        }

        private void InitializeSettingsGrid()
        {
            int count = 0;
            settingsGrid.RowDefinitions.Add(new RowDefinition());
            for (int i = 0; i < settingsGrid.Children.Count; i++)
            {
                FrameworkElement element = settingsGrid.Children[i] as FrameworkElement;
                Grid.SetRow(element, count);
                if (element is Grid)
                {
                    Grid.SetColumnSpan(element, 3);
                    count += 1;
                    settingsGrid.RowDefinitions.Add(new RowDefinition());
                }
                else if (!(element is TextBlock) && element.Tag == null)
                {
                    Grid.SetColumn(element, 1);
                    count += 1;
                    settingsGrid.RowDefinitions.Add(new RowDefinition());
                }
            }
        }

        private void reset_Click(object sender, RoutedEventArgs e)
        {
            InitializeChart();
            UpdateSeries();
        }

        private void ComboBox_DropDownOpened(object sender, EventArgs e)
        {
            fakeBox.Visibility = Visibility.Collapsed;
        }

        private void ComboBox_DropDownClosed(object sender, EventArgs e)
        {
            fakeBox.Visibility = Visibility.Visible;
            settingsBox.SelectedIndex = 0;
        }

        private void radioButtonChecked(object sender, RoutedEventArgs e)
        {
            e.Handled = true;
            string name = (sender as RadioButton).Name;
            if (name == "radio1")
                axesCollection = lineChart.XAxes;
            if (name == "radio2")
                axesCollection = lineChart.X2Axes;
            if (name == "radio3")
                axesCollection = lineChart.YAxes;
            if (name == "radio4")
                axesCollection = lineChart.Y2Axes;

            axisIndex.ItemsSource = axesCollection;
            axisIndex.SelectedIndex = 0;
        }

        private double MinX(int index)
        {
            return 0 + index * 1;
        }

        private double MaxX(int index)
        {
            return 10 + index * 1;
        }

        private double MinY(int index)
        {
            return 0 + index * 10;
        }

        private double MaxY(int index)
        {
            return 40 + index * 10;
        }

        private void axisIndex_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            int index = axesCollection.Count - 1;
            if (axisIndex.SelectedIndex != -1)
            {
                if (axesCollection == lineChart.XAxes || axesCollection == lineChart.X2Axes)
                {
                    double min = MinX(index);
                    double max = MaxX(index);

                    crossValue.Minimum = min;
                    crossValue.Maximum = max;
                    minValue.Minimum = min - 5;
                    minValue.Maximum = min + 5;
                    maxValue.Minimum = max - 5;
                    maxValue.Maximum = max + 5;
                    interval.Minimum = 1;
                    interval.Maximum = (max - min) * 2;
                }
                else
                {
                    double min = MinY(index);
                    double max = MaxY(index);

                    crossValue.Minimum = axesCollection[axisIndex.SelectedIndex].MinValue;
                    crossValue.Maximum = axesCollection[axisIndex.SelectedIndex].MaxValue;
                    minValue.Minimum = min - 20;
                    minValue.Maximum = min + 20;
                    maxValue.Minimum = max - 20;
                    maxValue.Maximum = max + 20;
                    interval.Minimum = 1;
                    interval.Maximum = max - min;
                }

                Binding binding = null;

                BindingOperations.ClearBinding(minValue, Slider.ValueProperty);
                binding = new Binding("MinValue");
                binding.Source = axesCollection[axisIndex.SelectedIndex];
                BindingOperations.SetBinding(minValue, Slider.ValueProperty, binding);

                BindingOperations.ClearBinding(maxValue, Slider.ValueProperty);
                binding = new Binding("MaxValue");
                binding.Source = axesCollection[axisIndex.SelectedIndex];
                BindingOperations.SetBinding(maxValue, Slider.ValueProperty, binding);

                BindingOperations.ClearBinding(interval, Slider.ValueProperty);
                binding = new Binding("IntervalCount");
                binding.Source = axesCollection[axisIndex.SelectedIndex];
                BindingOperations.SetBinding(interval, Slider.ValueProperty, binding);

                if (axisIndex.SelectedIndex == 0 &&
                    (axesCollection == lineChart.XAxes || axesCollection == lineChart.YAxes))
                {
                    crossValueTitle.Visibility = Visibility.Visible;
                    crossValue.Visibility = Visibility.Visible;

                    BindingOperations.ClearBinding(crossValue, Slider.ValueProperty);
                    binding = new Binding("AxisCrossingPoint");
                    binding.Source = axesCollection[axisIndex.SelectedIndex];
                    BindingOperations.SetBinding(crossValue, Slider.ValueProperty, binding);
                }
                else
                {
                    crossValueTitle.Visibility = Visibility.Collapsed;
                    crossValue.Visibility = Visibility.Collapsed;
                }
            }
        }

        private void seriesIndex_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            UpdateSeries();
        }

        private void xButton_Click(object sender, RoutedEventArgs e)
        {
            string s = xButton.Content.ToString();
            xButton.Content = s == "X" ? "X2" : "X";
            UpdateSeries();
        }

        private void yButton_Click(object sender, RoutedEventArgs e)
        {
            string s = yButton.Content.ToString();
            yButton.Content = s == "Y" ? "Y2" : "Y";
            UpdateSeries();
        }

        private void yAxisIndex_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            UpdateSeries();
        }

        private void xAxisIndex_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            UpdateSeries();
        }

        private void UpdateSeries()
        {
            if (xButton == null || yButton == null)
                return;

            string sx = xButton.Content.ToString();
            string sy = yButton.Content.ToString();
            AxesSeries series = lineChart.Series[seriesIndex.SelectedIndex] as AxesSeries;

            AxesCollection xAxisCollection = sx == "X" ? lineChart.XAxes : lineChart.X2Axes;
            AxesCollection yAxisCollection = sy == "Y" ? lineChart.YAxes : lineChart.Y2Axes;

            xAxisIndex.Maximum = xAxisCollection.Count - 1;
            yAxisIndex.Maximum = yAxisCollection.Count - 1;

            int xIndex = (int)Math.Min(xAxisIndex.Value, xAxisCollection.Count - 1);
            int yIndex = (int)Math.Min(yAxisIndex.Value, yAxisCollection.Count - 1);

            series.XAxis = xIndex > -1 ? xAxisCollection[xIndex] : null;
            series.YAxis = yIndex > -1 ? yAxisCollection[yIndex] : null;
        }

        private void addButton_Click(object sender, RoutedEventArgs e)
        {
            axesCollection.Add(CreateAxis(axesCollection));
        }

        private void removeButton_Click(object sender, RoutedEventArgs e)
        {
            if (axesCollection.Count > 0)
            {
                if (axisIndex.SelectedIndex == axesCollection.Count - 1)
                {
                    BindingOperations.ClearBinding(crossValue, Slider.ValueProperty);
                    BindingOperations.ClearBinding(minValue, Slider.ValueProperty);
                    BindingOperations.ClearBinding(maxValue, Slider.ValueProperty);
                    BindingOperations.ClearBinding(interval, Slider.ValueProperty);
                }
                Axis axis = axesCollection[axesCollection.Count - 1];
                axesCollection.Remove(axis);
                if (axisIndex.SelectedIndex == -1)
                    axisIndex.SelectedIndex = axisIndex.Items.Count - 1;
            }
        }

        private Axis CreateAxis(AxesCollection collection)
        {
            Axis axis = new Axis();
            int index = collection.Count;
            if (collection == lineChart.XAxes || collection == lineChart.X2Axes)
            {
                double min = MinX(index);
                double max = MaxX(index);

                axis.MinValue = min;
                axis.MaxValue = max;                
                axis.Title = collection == lineChart.XAxes ? "X-Axis " + index.ToString() : "X2-Axis " + index.ToString();
                axis.Interval = 1;
                axis.Tick = 3;
                axis.Tag = collection == lineChart.XAxes ? "X-Axis " + index.ToString() : "X2-Axis " + index.ToString();
                axis.IntervalCount = 10;
            }
            else
            {
                double min = MinY(index);
                double max = MaxY(index);

                axis.MinValue = min;
                axis.MaxValue = max;                
                axis.Title = collection == lineChart.YAxes ? "Y-Axis " + index.ToString() : "Y2-Axis " + index.ToString();
                axis.TitleRotationAngle = collection == lineChart.YAxes ? 270 : 90;
                axis.Interval = 10;
                axis.Tick = 3;
                axis.Tag = collection == lineChart.YAxes ? "Y-Axis " + index.ToString() : "Y2-Axis " + index.ToString();
                axis.IntervalCount = 4;
            }

            axis.LabelType = LabelType.AutoScale;
            return axis;
        }

        private AxesCollection axesCollection;
    }
}
